/*
============================================================================
Name        : generate_trunk_conf.sh
Author      : ssc
Version     : v1.0
Copyright   : ZYCOO copyright
Description : Generate trunk info from mysql to turnk conf file
============================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include <time.h>
#include <ctype.h>
#include <cjson/cJSON.h>

#include <mysql/mysql.h>

MYSQL *g_conn; // mysql 连接
MYSQL_RES *g_res; // mysql 记录集
MYSQL_ROW g_row; // 字符串数组，mysql 记录行

#define MAX_TRUNK_SIZE 256
#define MIDLE_SIZE 512
#define MINI_SIZE 64
#define MYSQL_CONNECT_CONF "/etc/asterisk/exten_gen.ini"
#define TRUNK_DST_FILE "/etc/asterisk/users_voiptrunk.conf"
#define TRUNK_GLOBAL_FILE "/etc/asterisk/extensions_trunks_global_custom.conf"
#define REGISTRATION_FILE "/etc/asterisk/sip_registrations.conf"
#define DID_CONF_FILE "/etc/asterisk/extensions_extendid_custom.conf"
#define DOD_CONF_FILE "/etc/asterisk/extensions_extendod_custom.conf"
#define KEYVALLEN 100
#define VERSION "V1.0.1"

#define QUERY_TRUNK_SQL "select trunkactive,trunkstyle,trunk,host,port,voipusername,authuser,\
fromuser,fromdomain,contact,voipsecret,insecure,calllimit,language,qualifyfreq,\
transport,hassip,registersip,hasiax,registeriax,context,hasexten,nat,directmedia\
,dtmfmode,videosupport,encryption,srtpcapable,prack,alaw,ulaw,g722,g729,g726,gsm\
,speex,h261,h263,h263p,h264,vp8,opus,outboundcid,trunkcid_preferred,prefix,qualify from t_pbx_users_voiptrunk"

#define QUERY_DID_SQL "select didnumber,type,exten from t_pbx_inbound_did"
#define QUERY_DOD_SQL "select dodexten,trunkname,trunk from v_extension_trunk_dod"

char g_host_name[MINI_SIZE];
char g_user_name[MINI_SIZE];
char g_password[MINI_SIZE];
char g_db_name[MINI_SIZE];
const unsigned int g_db_port = 3306;

//读取配置文件函数----功能：删除左边空格
char *l_trim(char *szOutput, const char *szInput)
{
    assert(szInput != NULL);
    assert(szOutput != NULL);
    assert(szOutput != szInput);
    for   (NULL; *szInput != '\0' && isspace(*szInput); ++szInput)
    {
        ;
    }
    return strcpy(szOutput, szInput);
}

/*   删除右边的空格   */
char *r_trim(char *szOutput, const char *szInput)
{
    char *p = NULL;
    assert(szInput != NULL);
    assert(szOutput != NULL);
    assert(szOutput != szInput);
    strcpy(szOutput, szInput);
    for(p = szOutput + strlen(szOutput) - 1; p >= szOutput && isspace(*p); --p)
    {
        ;
    }
    *(++p) = '\0';
    return szOutput;
}

/*   删除两边的空格   */
char *a_trim(char *szOutput, const char *szInput)
{
    char *p = NULL;
    assert(szInput != NULL);
    assert(szOutput != NULL);
    l_trim(szOutput, szInput);
    for   (p = szOutput + strlen(szOutput) - 1; p >= szOutput && isspace(*p); --p)
    {
        ;
    }
    *(++p) = '\0';
    return szOutput;
}

//main函数接口 参数1：配置文件路径 参数2：配置文件的那一部分，如general 参数3：键名 参数4：键值
int GetProfileString(char *profile, char *AppName, char *KeyName, char *KeyVal )
{
    char appname[32], keyname[32];
    char *buf, *c;
    char buf_i[KEYVALLEN], buf_o[KEYVALLEN];
    FILE *fp;
    int found = 0; /* 1 AppName 2 KeyName */
    if( (fp = fopen( profile, "r" )) == NULL )
    {
        printf( "openfile [%s] error [%s]\n", profile, strerror(errno) );
        return(-1);
    }
    fseek( fp, 0, SEEK_SET );
    memset( appname, 0, sizeof(appname) );
    sprintf( appname, "[%s]", AppName );

    while( !feof(fp) && fgets( buf_i, KEYVALLEN, fp ) != NULL )
    {
        l_trim(buf_o, buf_i);
        if( strlen(buf_o) <= 0 )
            continue;
        buf = NULL;
        buf = buf_o;

        if( found == 0 )
        {
            if( buf[0] != '[' )
            {
                continue;
            }
            else if ( strncmp(buf, appname, strlen(appname)) == 0 )
            {
                found = 1;
                continue;
            }

        }
        else if( found == 1 )
        {
            if( buf[0] == '#' )
            {
                continue;
            }
            else if ( buf[0] == '[' )
            {
                break;
            }
            else
            {
                if( (c = (char *)strchr(buf, '=')) == NULL )
                    continue;
                memset( keyname, 0, sizeof(keyname) );

                sscanf( buf, "%[^=|^ |^\t]", keyname );
                if( strcmp(keyname, KeyName) == 0 )
                {
                    sscanf( ++c, "%[^\n]", KeyVal );
                    char *KeyVal_o = (char *)malloc(strlen(KeyVal) + 1);
                    if(KeyVal_o != NULL)
                    {
                        memset(KeyVal_o, 0, sizeof(KeyVal_o));
                        a_trim(KeyVal_o, KeyVal);
                        if(KeyVal_o && strlen(KeyVal_o) > 0)
                            strcpy(KeyVal, KeyVal_o);
                        free(KeyVal_o);
                        KeyVal_o = NULL;
                    }
                    found = 2;
                    break;
                }
                else
                {
                    continue;
                }
            }
        }
    }
    fclose( fp );
    if( found == 2 )
        return(0);
    else
        return(-1);
}

char * mytime(){
        time_t my_time;
        time(&my_time);
        char *time_string = ctime(&my_time);
        if (time_string[strlen(time_string) - 1] == '\n')
        {
                time_string[strlen(time_string) - 1] = '\0';
        }
        return time_string;
}

void print_mysql_error(const char *msg) { // 打印最后一次错误
if (msg)
    printf("%s: %s\n", msg, mysql_error(g_conn));
else
    puts(mysql_error(g_conn));
}

int executesql(const char * sql) {
/*query the database according the sql*/
if (mysql_real_query(g_conn, sql, strlen(sql))) // 如果失败
    return -1; // 表示失败

return 0; // 成功执行
}


int init_mysql() { // 初始化连接
// init the database connection
g_conn = mysql_init(NULL);

/* connect the database */
if(!mysql_real_connect(g_conn, g_host_name, g_user_name, g_password, g_db_name, g_db_port, NULL, 0)) // 如果失败
    return -1;

// 是否连接已经可用
if (executesql("set names utf8")) // 如果失败
    return -1;

return 0; // 返回成功
}

// mysql> desc users_voiptrunk;
// +---------------+--------------+------+-----+---------------------+----------------+
// | Field         | Type         | Null | Key | Default             | Extra          |
// +---------------+--------------+------+-----+---------------------+----------------+
// | _id           | int(16)      | NO   | PRI | NULL                | auto_increment |
// | _trunkactive  | varchar(3)   | NO   |     | yes                 |                |
// | _trunk        | varchar(64)  | NO   | UNI |                     |                |
// | _trunkname    | varchar(64)  | NO   | UNI | NULL                |                |
// | _trunkstyle   | varchar(64)  | NO   |     |                     |                |
// | _host         | varchar(64)  | NO   |     | yes                 |                |
// | _port         | varchar(8)   | NO   |     |                     |                |
// | _voipusername | varchar(64)  | NO   |     |                     |                |
// | _authuser     | varchar(64)  | NO   |     |                     |                |
// | _fromuser     | varchar(64)  | NO   |     |                     |                |
// | _fromdomain   | varchar(64)  | NO   |     |                     |                |
// | _contact      | varchar(64)  | NO   |     |                     |                |
// | _voipsecret   | varchar(128) | NO   |     |                     |                |
// | _outboundcid  | varchar(32)  | NO   |     |                     |                |
// | _prefix       | varchar(16)  | NO   |     |                     |                |
// | _insecure     | varchar(16)  | NO   |     | port,invite         |                |
// | _calllimit    | varchar(3)   | NO   |     | 0                   |                |
// | _language     | varchar(8)   | NO   |     |                     |                |
// | _qualify      | varchar(8)   | NO   |     | 2000                |                |
// | _qualifyfreq  | varchar(8)   | NO   |     | 60                  |                |
// | _transport    | varchar(16)  | NO   |     | udp                 |                |
// | _faxdetect    | varchar(3)   | NO   |     | no                  |                |
// | _hassip       | varchar(3)   | NO   |     | yes                 |                |
// | _registersip  | varchar(3)   | NO   |     | no                  |                |
// | _hasiax       | varchar(3)   | NO   |     | no                  |                |
// | _registeriax  | varchar(3)   | NO   |     | no                  |                |
// | _context      | varchar(32)  | NO   |     | default             |                |
// | _hasexten     | varchar(3)   | NO   |     | yes                 |                |
// | _nat          | varchar(32)  | NO   |     | force_rport,comedia |                |
// | _directmedia  | varchar(16)  | NO   |     |                     |                |
// | _dtmfmode     | varchar(8)   | NO   |     | rfc2833             |                |
// | _videosupport | varchar(3)   | NO   |     | no                  |                |
// | _encryption   | varchar(3)   | NO   |     | yes                 |                |
// | _srtpcapable  | varchar(3)   | NO   |     | no                  |                |
// | _prack        | varchar(3)   | NO   |     | no                  |                |
// | _alaw         | int(1)       | NO   |     | 1                   |                |
// | _ulaw         | int(1)       | NO   |     | 1                   |                |
// | _g722         | int(1)       | NO   |     | 0                   |                |
// | _g729         | int(1)       | NO   |     | 0                   |                |
// | _g726         | int(1)       | NO   |     | 0                   |                |
// | _gsm          | int(1)       | NO   |     | 0                   |                |
// | _speex        | int(1)       | NO   |     | 0                   |                |
// | _h261         | int(1)       | NO   |     | 0                   |                |
// | _h263         | int(1)       | NO   |     | 0                   |                |
// | _h263p        | int(1)       | NO   |     | 0                   |                |
// | _h264         | int(1)       | NO   |     | 0                   |                |
// | _vp8          | int(1)       | YES  |     | 0                   |                |
// | _opus         | int(1)       | YES  |     | 0                   |                |
// | _position     | varchar(256) | NO   |     |                     |                |
// | _recordin     | varchar(32)  | NO   |     |                     |                |
// | _recordout    | varchar(32)  | NO   |     |                     |                |
// | _F1           | varchar(128) | NO   |     |                     |                |
// | _F2           | varchar(128) | NO   |     |                     |                |
// | _F3           | varchar(128) | NO   |     |                     |                |
// | _F4           | varchar(128) | NO   |     |                     |                |
// +---------------+--------------+------+-----+---------------------+----------------+
// 55 rows in set (0.00 sec)

typedef struct trunk {
    char trunkactive[MINI_SIZE]; //0 //*
    char trunkstyle[MINI_SIZE]; //1 //* 
    char trunk[MINI_SIZE]; //2 //*
    char host[MINI_SIZE]; //3 //*
    char port[MINI_SIZE]; //4 //*
    char voipusername[MINI_SIZE]; //5 //*
    char authuser[MINI_SIZE]; //6
    char fromuser[MINI_SIZE]; //7
    char fromdomain[MINI_SIZE]; //8
    char contact[MINI_SIZE]; //9
    char voipsecret[MINI_SIZE]; //10 //*
    char insecure[MINI_SIZE]; //11
    char calllimit[MINI_SIZE]; //12
    char language[MINI_SIZE]; //13
    char qualifyfreq[MINI_SIZE]; //14
    char transport[MINI_SIZE]; //15
    char hassip[MINI_SIZE]; //16
    char registersip[MINI_SIZE]; //17
    char hasiax[MINI_SIZE]; //18
    char registeriax[MINI_SIZE]; //19
    char context[MINI_SIZE]; //20 //*
    char hasexten[MINI_SIZE]; //21
    char nat[MINI_SIZE]; //22
    char directmedia[MINI_SIZE]; //23
    char dtmfmode[MINI_SIZE]; //24
    char videosupport[MINI_SIZE]; //25
    char encryption[MINI_SIZE]; //26
    char srtpcapable[MINI_SIZE]; //27
    char prack[MINI_SIZE]; //28
    char alaw[MINI_SIZE]; //29
    char ulaw[MINI_SIZE]; //30
    char g722[MINI_SIZE]; //31
    char g729[MINI_SIZE]; //32
    char g726[MINI_SIZE]; //33
    char gsm[MINI_SIZE]; //34
    char speex[MINI_SIZE]; //35
    char h261[MINI_SIZE]; //36
    char h263[MINI_SIZE]; //37
    char h263p[MINI_SIZE]; //38
    char h264[MINI_SIZE]; //39
    char vp8[MINI_SIZE]; //40
    char opus[MINI_SIZE]; //41
    char outboundcid[MINI_SIZE]; //42
    char trunkcid_preferred[MINI_SIZE]; //43
    char prefix[MINI_SIZE]; //44
    char qualify[MINI_SIZE]; //45
    char allow[MIDLE_SIZE];
} TrunkObject;

typedef struct did {
    char didnumber[MINI_SIZE];
    char type[MINI_SIZE];
    char exten[MINI_SIZE];
} DidObject;

typedef struct dod {
    char dodexten[MINI_SIZE];
    char trunkname[MINI_SIZE];
    char trunk[MINI_SIZE];
} DodObject;

int main(int argc, char **argv) {

if (argc == 2){
    if (strcmp(argv[1], "-v") == 0)
    {
        printf("%s: %s\n", argv[0], VERSION);
        return EXIT_SUCCESS;
    }else if (strcmp(argv[1], "trunk") == 0 || strcmp(argv[1], "did") == 0 || strcmp(argv[1], "dod") == 0){
    }else{
        printf("Usage:%s -v | trunk | did | dod\n", argv[0]);
        return EXIT_SUCCESS;
    }
}else{
    printf("Usage:%s -v | trunk | did | dod\n", argv[0]);
    return EXIT_SUCCESS;
}
/*
memset(g_host_name, 0, sizeof(g_host_name));
memset(g_user_name, 0, sizeof(g_user_name));
memset(g_password, 0, sizeof(g_password));
memset(g_db_name, 0, sizeof(g_db_name));

GetProfileString(MYSQL_CONNECT_CONF, "general", "dbserverip", g_host_name);
GetProfileString(MYSQL_CONNECT_CONF, "general", "dbuser", g_user_name);
GetProfileString(MYSQL_CONNECT_CONF, "general", "dbpasswd", g_password);
GetProfileString(MYSQL_CONNECT_CONF, "general", "dbname", g_db_name);
*/
strcpy(g_host_name,getenv("MYSQL"));
strcpy(g_user_name,getenv("MYSQL_USER"));
strcpy(g_password,getenv("MYSQL_PASSWORD"));
strcpy(g_db_name,getenv("MYSQL_DATABASE"));

if (init_mysql()){
    print_mysql_error(NULL);
    exit(1);
}
if (strcmp(argv[1], "trunk") == 0){
    if (executesql(QUERY_TRUNK_SQL)){
        print_mysql_error(NULL);
        exit(1);
    }

    g_res = mysql_store_result(g_conn); // 从服务器传送结果集至本地，mysql_use_result直接使用服务器上的记录集

    // int iNum_rows = mysql_num_rows(g_res); // 得到记录的行数
    // int iNum_fields = mysql_num_fields(g_res); // 得到记录的列数

    TrunkObject trunkObject[MAX_TRUNK_SIZE];
    memset(trunkObject, 0, sizeof(trunkObject));

    char registrationString[MIDLE_SIZE];

    int trunk_size = 0;

    while ((g_row=mysql_fetch_row(g_res))){ // 打印结果集
        // printf("%s\t%s\n", g_row[0], g_row[1]); // 第一，第二字段
        if (g_row[0] == NULL || g_row[1] == NULL || g_row[2] == NULL || g_row[3] == NULL \
            || g_row[4] == NULL || g_row[5] == NULL || g_row[10] == NULL || g_row[20] == NULL \
            || strcmp(g_row[0], "yes") != 0){
            // printf("%s\n%s\n%s\n%s\n%s\n%s\n%s\n%s\n", g_row[0], g_row[1], g_row[2], g_row[3], g_row[4], g_row[5], g_row[10], g_row[20]);
            printf("some feild is empty!\n");
            continue;
        }

        // trunkactive,trunkstyle,trunk,host,port,voipusername,authuser,\
    fromuser,fromdomain,contact,voipsecret,insecure,calllimit,language,qualifyfreq,\
    transport,hassip,registersip,hasiax,registeriax,context,hasexten,nat,directmedia\
    ,dtmfmode,videosupport,encryption,srtpcapable,prack,alaw,ulaw,g722,g729,g726,gsm\
    ,speex,h261,h263,h263p,h264,vp8,opus

        strcpy(trunkObject[trunk_size].trunkactive, g_row[0]);
        strcpy(trunkObject[trunk_size].trunkstyle, g_row[1]);
        strcpy(trunkObject[trunk_size].trunk, g_row[2]);
        strcpy(trunkObject[trunk_size].host, g_row[3]);
        strcpy(trunkObject[trunk_size].port, g_row[4]);
        strcpy(trunkObject[trunk_size].voipusername, g_row[5]);

        if (g_row[6] != NULL){
            strcpy(trunkObject[trunk_size].authuser, g_row[6]);
        }else{
            strcpy(trunkObject[trunk_size].authuser, "");
        }
        
        if (g_row[6] != NULL){
            strcpy(trunkObject[trunk_size].authuser, g_row[6]);
        }else{
            strcpy(trunkObject[trunk_size].authuser, "");
        }

        if (g_row[7] != NULL){
            strcpy(trunkObject[trunk_size].fromuser, g_row[7]);
        }else{
            strcpy(trunkObject[trunk_size].fromuser, "");
        }

        if (g_row[8] != NULL){
            strcpy(trunkObject[trunk_size].fromdomain, g_row[8]);
        }else{
            strcpy(trunkObject[trunk_size].fromdomain, "");
        }

        if (g_row[9] != NULL || strcmp(g_row[9], "") == 0){
            strcpy(trunkObject[trunk_size].contact, g_row[9]);
        }else{
            strcpy(trunkObject[trunk_size].contact, trunkObject[trunk_size].voipusername);
        }

        strcpy(trunkObject[trunk_size].voipsecret, g_row[10]);

        if (g_row[11] != NULL){
            strcpy(trunkObject[trunk_size].insecure, g_row[11]);
        }else{
            strcpy(trunkObject[trunk_size].insecure, "");
        }
        
        if (g_row[12] != NULL){
            strcpy(trunkObject[trunk_size].calllimit, g_row[12]);
        }else{
            strcpy(trunkObject[trunk_size].calllimit, "");
        }

        if (g_row[13] != NULL || strcmp(g_row[13], "") == 0){
            strcpy(trunkObject[trunk_size].language, g_row[13]);
        }else{
            strcpy(trunkObject[trunk_size].language, "cn");
        }

        if (g_row[14] != NULL){
            strcpy(trunkObject[trunk_size].qualifyfreq, g_row[14]);
        }else{
            strcpy(trunkObject[trunk_size].qualifyfreq, "");
        }

        if (g_row[15] != NULL){
            strcpy(trunkObject[trunk_size].transport, g_row[15]);
        }else{
            strcpy(trunkObject[trunk_size].transport, "");
        }

        if (g_row[16] != NULL){
            strcpy(trunkObject[trunk_size].hassip, g_row[16]);
        }else{
            strcpy(trunkObject[trunk_size].hassip, "");
        }

        if (g_row[17] != NULL){
            strcpy(trunkObject[trunk_size].registersip, g_row[17]);
        }else{
            strcpy(trunkObject[trunk_size].registersip, "");
        }

        if (g_row[18] != NULL){
            strcpy(trunkObject[trunk_size].hasiax, g_row[18]);
        }else{
            strcpy(trunkObject[trunk_size].hasiax, "");
        }

        if (g_row[19] != NULL){
            strcpy(trunkObject[trunk_size].registeriax, g_row[19]);
        }else{
            strcpy(trunkObject[trunk_size].registeriax, "");
        }

        strcpy(trunkObject[trunk_size].context, g_row[20]);

        if (g_row[21] != NULL){
            strcpy(trunkObject[trunk_size].hasexten, g_row[21]);
        }else{
            strcpy(trunkObject[trunk_size].hasexten, "");
        }
        
        if (g_row[22] != NULL){
            strcpy(trunkObject[trunk_size].nat, g_row[22]);
        }else{
            strcpy(trunkObject[trunk_size].nat, "");
        }

        if (g_row[23] != NULL){
            strcpy(trunkObject[trunk_size].directmedia, g_row[23]);
        }else{
            strcpy(trunkObject[trunk_size].directmedia, "");
        }

        if (g_row[24] != NULL){
            strcpy(trunkObject[trunk_size].dtmfmode, g_row[24]);
        }else{
            strcpy(trunkObject[trunk_size].dtmfmode, "");
        }

        if (g_row[25] != NULL){
            strcpy(trunkObject[trunk_size].videosupport, g_row[25]);
        }else{
            strcpy(trunkObject[trunk_size].videosupport, "");
        }

        if (g_row[26] != NULL){
            strcpy(trunkObject[trunk_size].encryption, g_row[26]);
        }else{
            strcpy(trunkObject[trunk_size].encryption, "");
        }

        if (g_row[27] != NULL){
            strcpy(trunkObject[trunk_size].srtpcapable, g_row[27]);
        }else{
            strcpy(trunkObject[trunk_size].srtpcapable, "");
        }

        if (g_row[28] != NULL){
            strcpy(trunkObject[trunk_size].prack, g_row[28]);
        }else{
            strcpy(trunkObject[trunk_size].prack, "");
        }

        if (g_row[29] != NULL){
            strcpy(trunkObject[trunk_size].alaw, g_row[29]);
        }else{
            strcpy(trunkObject[trunk_size].alaw, "");
        }

        if (g_row[30] != NULL){
            strcpy(trunkObject[trunk_size].ulaw, g_row[30]);
        }else{
            strcpy(trunkObject[trunk_size].ulaw, "");
        }
        
        if (g_row[31] != NULL){
            strcpy(trunkObject[trunk_size].g722, g_row[31]);
        }else{
            strcpy(trunkObject[trunk_size].g722, "");
        }

        if (g_row[32] != NULL){
            strcpy(trunkObject[trunk_size].g729, g_row[32]);
        }else{
            strcpy(trunkObject[trunk_size].g729, "");
        }

        if (g_row[33] != NULL){
            strcpy(trunkObject[trunk_size].g726, g_row[33]);
        }else{
            strcpy(trunkObject[trunk_size].g726, "");
        }

        if (g_row[34] != NULL){
            strcpy(trunkObject[trunk_size].gsm, g_row[34]);
        }else{
            strcpy(trunkObject[trunk_size].gsm, "");
        }

        if (g_row[35] != NULL){
            strcpy(trunkObject[trunk_size].speex, g_row[35]);
        }else{
            strcpy(trunkObject[trunk_size].speex, "");
        }

        if (g_row[36] != NULL){
            strcpy(trunkObject[trunk_size].h261, g_row[36]);
        }else{
            strcpy(trunkObject[trunk_size].h261, "");
        }

        if (g_row[37] != NULL){
            strcpy(trunkObject[trunk_size].h263, g_row[37]);
        }else{
            strcpy(trunkObject[trunk_size].h263, "");
        }

        if (g_row[38] != NULL){
            strcpy(trunkObject[trunk_size].h263p, g_row[38]);
        }else{
            strcpy(trunkObject[trunk_size].h263p, "");
        }

        if (g_row[39] != NULL){
            strcpy(trunkObject[trunk_size].h264, g_row[39]);
        }else{
            strcpy(trunkObject[trunk_size].h264, "");
        }
        
        if (g_row[40] != NULL){
            strcpy(trunkObject[trunk_size].vp8, g_row[40]);
        }else{
            strcpy(trunkObject[trunk_size].vp8, "");
        }

        if (g_row[41] != NULL){
            strcpy(trunkObject[trunk_size].opus, g_row[41]);
        }else{
            strcpy(trunkObject[trunk_size].opus, "");
        }

        if (g_row[42] != NULL){
            strcpy(trunkObject[trunk_size].outboundcid, g_row[42]);
        }else{
            strcpy(trunkObject[trunk_size].outboundcid, "");
        }

        if (g_row[43] != NULL){
            strcpy(trunkObject[trunk_size].trunkcid_preferred, g_row[43]);
        }else{
            strcpy(trunkObject[trunk_size].trunkcid_preferred, "");
        }

        if (g_row[44] != NULL){
            strcpy(trunkObject[trunk_size].prefix, g_row[44]);
        }else{
            strcpy(trunkObject[trunk_size].prefix, "");
        }

        if (g_row[45] != NULL){
            strcpy(trunkObject[trunk_size].qualify, g_row[45]);
        }else{
            strcpy(trunkObject[trunk_size].qualify, "");
        }

        if (strcmp(trunkObject[trunk_size].alaw, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "alaw,");
        }

        if (strcmp(trunkObject[trunk_size].ulaw, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sulaw,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].g722, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sg722,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].g729, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sg729,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].g726, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sg726,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].speex, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sspeex,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].gsm, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sgsm,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].h261, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sh261,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].h263, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sh263,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].h263p, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sh263p,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].h264, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sh264,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].vp8, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%svp8,", trunkObject[trunk_size].allow);
        }

        if (strcmp(trunkObject[trunk_size].opus, "1") == 0){
            sprintf(trunkObject[trunk_size].allow, "%sopus,", trunkObject[trunk_size].allow);
        }
        // printf("trunk:%s\n", trunkObject[trunk_size].trunk);
        trunkObject[trunk_size].allow[strlen(trunkObject[trunk_size].allow)-1]='\0';
        // printf("allow:%s\n", trunkObject[trunk_size].allow);
        trunk_size++;
    }

    // printf("trunk_size:%d\n", trunk_size);

    FILE *conf_fp = fopen(TRUNK_DST_FILE, "w+");
    FILE *registration_fp = fopen(REGISTRATION_FILE, "w+");

    if (conf_fp == NULL){
        perror("Open trunk conf file Error: ");
        exit(1);
    }

    fprintf(conf_fp, ";!\n\
;! Automatically generated configuration file\n\
;! Filename: users_voiptrunk.conf (/etc/asterisk/users_voiptrunk.conf)\n\
;! Generator: Generator Trunk\n\
;! Creation Date: %s\n\
;!\n\n\
",\
mytime()\
);

    if (registration_fp == NULL){
        perror("Open sip registration conf file Error: ");
        exit(1);
    }

    fprintf(registration_fp, ";!\n\
;! Automatically generated configuration file\n\
;! Filename: sip_registrations.conf (/etc/asterisk/sip_registrations.conf)\n\
;! Generator: Generator Trunk\n\
;! Creation Date: %s\n\
;!\n\n\
",\
mytime()\
);

    int i = 0;

    while(i < trunk_size){
        if(strcmp(trunkObject[i].trunkactive, "yes") == 0){
            if (strcmp(trunkObject[i].hassip, "yes") == 0){
                fprintf(conf_fp, "\
[%s]\n\
host = %s\n\
port = %s\n\
username = %s\n\
authuser = %s\n\
fromuser = %s\n\
fromdomain = %s\n\
secret = %s\n\
insecure = %s\n\
call-limit = %s\n\
qualify = %s\n\
qualifyfreq = %s\n\
transport = %s\n\
hassip = %s\n\
registersip = %s\n\
hasiax = %s\n\
registeriax = %s\n\
context = from-voip\n\
hasexten = %s\n\
nat = %s\n\
directmedia = %s\n\
dtmfmode = %s\n\
videosupport = %s\n\
encryption = %s\n\
srtpcapable = %s\n\
prack = %s\n\
disallow = all\n\
allow = %s\
\n", \
trunkObject[i].trunk,\
trunkObject[i].host,\
trunkObject[i].port,\
trunkObject[i].voipusername,\
trunkObject[i].authuser,\
trunkObject[i].fromuser,\
trunkObject[i].fromdomain,\
trunkObject[i].voipsecret,\
trunkObject[i].insecure,\
trunkObject[i].calllimit,\
trunkObject[i].qualify,\
trunkObject[i].qualifyfreq,\
trunkObject[i].transport,\
trunkObject[i].hassip,\
trunkObject[i].registersip,\
trunkObject[i].hasiax,\
trunkObject[i].registeriax,\
trunkObject[i].hasexten,\
trunkObject[i].nat,\
trunkObject[i].directmedia,\
trunkObject[i].dtmfmode,\
trunkObject[i].videosupport,\
trunkObject[i].encryption,\
trunkObject[i].srtpcapable,\
trunkObject[i].prack,\
trunkObject[i].allow\
);
                if (strcmp(trunkObject[i].contact, "") != 0){
                    fprintf(conf_fp, "contact = %s\n", trunkObject[i].contact);
                }

                if (strcmp(trunkObject[i].language, "") != 0){
                    fprintf(conf_fp, "language = %s\n", trunkObject[i].language);
                }

                fprintf(conf_fp, "\n");

            }else if (strcmp(trunkObject[i].hasiax, "yes") == 0){
                fprintf(conf_fp, "\
[%s]\n\
host = %s\n\
port = %s\n\
username = %s\n\
secret = %s\n\
call-limit = %s\n\
qualify = %s\n\
hassip = %s\n\
registersip = %s\n\
hasiax = %s\n\
registeriax = %s\n\
context = %s\n\
disallow = all\n\
allow = %s\
\n", \
trunkObject[i].trunk,\
trunkObject[i].host,\
trunkObject[i].port,\
trunkObject[i].voipusername,\
trunkObject[i].voipsecret,\
trunkObject[i].calllimit,\
trunkObject[i].qualify,\
trunkObject[i].hassip,\
trunkObject[i].registersip,\
trunkObject[i].hasiax,\
trunkObject[i].registeriax,\
trunkObject[i].context,\
trunkObject[i].allow\
);
                if (strcmp(trunkObject[i].language, "") != 0){
                    fprintf(conf_fp, "language = %s\n", trunkObject[i].language);
                }

                fprintf(conf_fp, "\n");
            }

            if (strcmp(trunkObject[i].hassip, "yes") == 0){
                if (strcmp(trunkObject[i].hassip, "") != 0 && strcmp(trunkObject[i].host, "") != 0 && strcmp(trunkObject[i].voipusername, "") != 0){
                    memset(registrationString, 0, sizeof(registrationString));

                    if (strcmp(trunkObject[i].fromdomain, "") == 0 || strstr(trunkObject[i].authuser, "@") != NULL){
                        // register="${USERNAME}@${FROMDOMAIN}:${SECRET}${AUTHUSER}@${HOST}:${PORT}"
                        sprintf(registrationString, "%s:%s:%s@%s:%s", trunkObject[i].voipusername, trunkObject[i].voipsecret, \
                                                                    trunkObject[i].authuser, trunkObject[i].host, \
                                                                    trunkObject[i].port);
                        // printf("%d\nregistrationString:%s\n", __LINE__, registrationString);
                    }else{
                        // register="${USERNAME}:${SECRET}${AUTHUSER}@${HOST}:${PORT}"
                        sprintf(registrationString, "%s@%s:%s:%s@%s:%s", trunkObject[i].voipusername, trunkObject[i].fromdomain, \
                                                                        trunkObject[i].voipsecret, trunkObject[i].authuser, \
                                                                        trunkObject[i].host, trunkObject[i].port);
                    }

                    if (strcmp(trunkObject[i].contact, "") != 0){
                        strcat(registrationString,"/");
                        strcat(registrationString,trunkObject[i].contact);
                        //sprintf(registrationString, "%s/%s", registrationString, trunkObject[i].contact);
                    }
                    if(strcmp(trunkObject[i].trunkstyle, "Peer") != 0 && strlen(trunkObject[i].voipusername) > 0){
                        if (strcmp(trunkObject[i].transport, "tcp") == 0){
                            fprintf(registration_fp, "register=tcp://%s\n", registrationString);
                        }else if (strcmp(trunkObject[i].transport, "tls") == 0){
                            fprintf(registration_fp, "register=tls://%s\n", registrationString);
                        }else{
                            fprintf(registration_fp, "register=%s\n", registrationString);
                            // printf("%d\nregistrationString:%s\n", __LINE__, registrationString);
                        }
                    }
                }
            }
        }
        i++;  
    }

    fclose(conf_fp);
    fclose(registration_fp);

    FILE *global_fp = fopen(TRUNK_GLOBAL_FILE, "w+");

    if (global_fp == NULL){
        perror("Open trunk conf file Error: ");
        exit(1);
    }

    fprintf(global_fp, ";!\n\
;! Automatically generated configuration file\n\
;! Filename: extensions_trunks_global_custom.conf (/etc/asterisk/extensions_trunks_global_custom.conf)\n\
;! Generator: Generator Trunk\n\
;! Creation Date: %s\n\
;!\n\n\
",\
mytime()\
);

    i = 0;

    // CID_$trunk = $outboundcid
    // P_CID_$trunk=$trunkcid_preferred
    // $trunk=SIP/$trunk/$prefix
    if (trunk_size > 0){
        while(i < trunk_size){
            if (strcmp(trunkObject[i].hassip, "yes") == 0){
                fprintf(global_fp, "\
CID_%s = %s\n\
P_CID_%s = %s\n\
%s=SIP/%s/%s\
\n", \
trunkObject[i].trunk,\
trunkObject[i].outboundcid,\
trunkObject[i].trunk,\
trunkObject[i].trunkcid_preferred,\
trunkObject[i].trunk,\
trunkObject[i].trunk,\
trunkObject[i].prefix\
);
            }
                
            i++;
        }
    }

    fclose(global_fp);
}else if (strcmp(argv[1], "did") == 0){
    if (executesql(QUERY_DID_SQL)){
        print_mysql_error(NULL);
        exit(1);
    }

    g_res = mysql_store_result(g_conn);

    DidObject didObject[MAX_TRUNK_SIZE];
    memset(didObject, 0, sizeof(didObject));

    int did_size = 0;

    while ((g_row=mysql_fetch_row(g_res))){ // 打印结果集
        if (g_row[0] != NULL){
            strcpy(didObject[did_size].didnumber, g_row[0]);
        }else{
            strcpy(didObject[did_size].didnumber, "");
        }

        if (g_row[1] != NULL){
            strcpy(didObject[did_size].type, g_row[1]);
        }else{
            strcpy(didObject[did_size].type, "");
        }

        if (g_row[2] != NULL){
            strcpy(didObject[did_size].exten, g_row[2]);
        }else{
            strcpy(didObject[did_size].exten, "");
        }

        did_size++;
    }

    FILE *did_fp = fopen(DID_CONF_FILE, "w+");

    if (did_fp == NULL){
        perror("Open did conf file Error: ");
        exit(1);
    }

    fprintf(did_fp, ";!\n\
;! Automatically generated configuration file\n\
;! Filename: extensions_extendid_custom.conf (/etc/asterisk/extensions_extendid_custom.conf)\n\
;! Generator: Generator did\n\
;! Creation Date: %s\n\
;!\n\n\
",\
mytime()\
);

    int m = 0;

    if (did_size > 0){
        while(m < did_size){
            if (strcmp(didObject[m].didnumber, "") != 0 && strcmp(didObject[m].type, "") != 0){

                //exten => _didnumber,1,Goto(default,_destexten,1)
                if(strcmp(didObject[m].type, "extension") == 0){
                fprintf(did_fp, "\
exten => %s,1,Goto(default,%s,1)\
\n", \
didObject[m].didnumber,\
didObject[m].exten\
);
                }else if(strcmp(didObject[m].type, "ivr") == 0){
                fprintf(did_fp, "\
exten => %s,1,Goto(voicemenu-custom-%s,%s,1)\
\n", \
didObject[m].didnumber,\
didObject[m].exten,\
didObject[m].exten\
);
                }else if(strcmp(didObject[m].type, "group") == 0){
                fprintf(did_fp, "\
exten => %s,1,Goto(paging-group-%s,%s,1)\
\n", \
didObject[m].didnumber,\
didObject[m].exten,\
didObject[m].exten\
);
                }else if(strcmp(didObject[m].type, "trigger") == 0){
                fprintf(did_fp, "\
exten => %s,1,Goto(call-trigger,${EXTEN},1)\
\n", \
didObject[m].didnumber\
);
                }else if(strcmp(didObject[m].type, "user") == 0){
                int id = 100000 + atoi(didObject[m].exten);
                fprintf(did_fp, "\
exten => %s,1,Goto(manager-queue-%d,s,1)\
\n", \
didObject[m].didnumber,\
id\
);
                }
            }
            m++;
        }
    }

    fclose(did_fp);

}else if (strcmp(argv[1], "dod") == 0){
    if (executesql(QUERY_DOD_SQL)){
        print_mysql_error(NULL);
        exit(1);
    }

    g_res = mysql_store_result(g_conn);

    DodObject dodObject[MAX_TRUNK_SIZE];
    memset(dodObject, 0, sizeof(dodObject));

    int dod_size = 0;

    while ((g_row=mysql_fetch_row(g_res))){ // 打印结果集
        if (g_row[0] != NULL){
            strcpy(dodObject[dod_size].dodexten, g_row[0]);
        }else{
            strcpy(dodObject[dod_size].dodexten, "");
        }

        if (g_row[1] != NULL){
            strcpy(dodObject[dod_size].trunkname, g_row[1]);
        }else{
            strcpy(dodObject[dod_size].trunkname, "");
        }

        if (g_row[2] != NULL){
            strcpy(dodObject[dod_size].trunk, g_row[2]);
        }else{
            strcpy(dodObject[dod_size].trunk, "");
        }

        dod_size++;
    }

    FILE *dod_fp = fopen(DOD_CONF_FILE, "w+");

    if (dod_fp == NULL){
        perror("Open dod conf file Error: ");
        exit(1);
    }

    fprintf(dod_fp, ";!\n\
;! Automatically generated configuration file\n\
;! Filename: extensions_extendod_custom.conf (/etc/asterisk/extensions_extendod_custom.conf)\n\
;! Generator: Generator dod\n\
;! Creation Date: %s\n\
;!\n\n\
",\
mytime()\
);

    int n = 0;

    if (dod_size > 0){
        while(n < dod_size){
            if (strcmp(dodObject[n].dodexten, "") != 0 && strcmp(dodObject[n].trunk, "") != 0){

                //exten => _X./$dodexten,1, Macro(trunkdial-failover,,,${EXTEN},$trunk${EXTEN:0})

                fprintf(dod_fp, "\
exten => _X./%s,1, Macro(trunkdial-failover,,,${EXTEN},${%s}${EXTEN:0})\
\n", \
dodObject[n].dodexten,\
dodObject[n].trunk\
);
            }
            n++;
        }
    }

    fclose(dod_fp);
}else{
    printf("Usage:%s [-v] | trunk | did | dod\n", argv[0]);
}
mysql_free_result(g_res); // 释放结果集

mysql_close(g_conn); // 关闭链接

return EXIT_SUCCESS;

}