/*
============================================================================
Name        : generate_paging_conf.sh
Author      : ssc
Version     : v1.0
Copyright   : ZYCOO copyright
Description : Generate paging info from mysql to paging conf file
============================================================================
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>
#include <time.h>
#include <ctype.h>
#include <cjson/cJSON.h>

#include <mysql/mysql.h>

MYSQL *g_conn; // mysql 连接
MYSQL_RES *g_res; // mysql group记录集
MYSQL_ROW g_row; // 字符串数组，mysql 记录行
MYSQL_RES *d_res; // mysql device记录集
MYSQL_ROW d_row; // 字符串数组，mysql 记录行

#define MAX_TRUNK_SIZE 256
#define MAX_SIZE 2048
#define MIDLE_SIZE 512
#define MINI_SIZE 64
#define MYSQL_CONNECT_CONF "/etc/asterisk/exten_gen.ini"
#define EXTEN_USER_QUEUE_FILE "/etc/asterisk/extensions_users_queue_custom.conf"
#define QUEUES_FILE "/etc/asterisk/queues.conf"
#define KEYVALLEN 100
#define VERSION "V1.0.1"

#define QUERY_PAGING_USER_SQL "select id,phones,strategy,ring_duration,noanswer_dest from t_paging_users"

char g_host_name[MINI_SIZE];
char g_user_name[MINI_SIZE];
char g_password[MINI_SIZE];
char g_db_name[MINI_SIZE];
const unsigned int g_db_port = 3306;
char sql_tmp[MIDLE_SIZE];
char exten_tmp[MAX_SIZE];

//读取配置文件函数----功能：删除左边空格
char *l_trim(char *szOutput, const char *szInput)
{
    assert(szInput != NULL);
    assert(szOutput != NULL);
    assert(szOutput != szInput);
    for   (NULL; *szInput != '\0' && isspace(*szInput); ++szInput)
    {
        ;
    }
    return strcpy(szOutput, szInput);
}

//   删除右边的空格   
char *r_trim(char *szOutput, const char *szInput)
{
    char *p = NULL;
    assert(szInput != NULL);
    assert(szOutput != NULL);
    assert(szOutput != szInput);
    strcpy(szOutput, szInput);
    for(p = szOutput + strlen(szOutput) - 1; p >= szOutput && isspace(*p); --p)
    {
        ;
    }
    *(++p) = '\0';
    return szOutput;
}

//   删除两边的空格   
char *a_trim(char *szOutput, const char *szInput)
{
    char *p = NULL;
    assert(szInput != NULL);
    assert(szOutput != NULL);
    l_trim(szOutput, szInput);
    for   (p = szOutput + strlen(szOutput) - 1; p >= szOutput && isspace(*p); --p)
    {
        ;
    }
    *(++p) = '\0';
    return szOutput;
}
//main函数接口 参数1：配置文件路径 参数2：配置文件的那一部分，如general 参数3：键名 参数4：键值
int GetProfileString(char *profile, char *AppName, char *KeyName, char *KeyVal )
{
    char appname[32], keyname[32];
    char *buf, *c;
    char buf_i[KEYVALLEN], buf_o[KEYVALLEN];
    FILE *fp;
    int found = 0; /* 1 AppName 2 KeyName */
    if( (fp = fopen( profile, "r" )) == NULL )
    {
        printf( "openfile [%s] error [%s]\n", profile, strerror(errno) );
        return(-1);
    }
    fseek( fp, 0, SEEK_SET );
    memset( appname, 0, sizeof(appname) );
    sprintf( appname, "[%s]", AppName );

    while( !feof(fp) && fgets( buf_i, KEYVALLEN, fp ) != NULL )
    {
        l_trim(buf_o, buf_i);
        if( strlen(buf_o) <= 0 )
            continue;
        buf = NULL;
        buf = buf_o;

        if( found == 0 )
        {
            if( buf[0] != '[' )
            {
                continue;
            }
            else if ( strncmp(buf, appname, strlen(appname)) == 0 )
            {
                found = 1;
                continue;
            }

        }
        else if( found == 1 )
        {
            if( buf[0] == '#' )
            {
                continue;
            }
            else if ( buf[0] == '[' )
            {
                break;
            }
            else
            {
                if( (c = (char *)strchr(buf, '=')) == NULL )
                    continue;
                memset( keyname, 0, sizeof(keyname) );

                sscanf( buf, "%[^=|^ |^\t]", keyname );
                if( strcmp(keyname, KeyName) == 0 )
                {
                    sscanf( ++c, "%[^\n]", KeyVal );
                    char *KeyVal_o = (char *)malloc(strlen(KeyVal) + 1);
                    if(KeyVal_o != NULL)
                    {
                        memset(KeyVal_o, 0, sizeof(KeyVal_o));
                        a_trim(KeyVal_o, KeyVal);
                        if(KeyVal_o && strlen(KeyVal_o) > 0)
                            strcpy(KeyVal, KeyVal_o);
                        free(KeyVal_o);
                        KeyVal_o = NULL;
                    }
                    found = 2;
                    break;
                }
                else
                {
                    continue;
                }
            }
        }
    }
    fclose( fp );
    if( found == 2 )
        return(0);
    else
        return(-1);
}

char * mytime(){
        time_t my_time;
        time(&my_time);
        char *time_string = ctime(&my_time);
        if (time_string[strlen(time_string) - 1] == '\n')
        {
                time_string[strlen(time_string) - 1] = '\0';
        }
        return time_string;
}

void print_mysql_error(const char *msg) { // 打印最后一次错误
if (msg)
    printf("%s: %s\n", msg, mysql_error(g_conn));
else
    puts(mysql_error(g_conn));
}

int executesql(const char * sql) {
/*query the database according the sql*/
if (mysql_real_query(g_conn, sql, strlen(sql))) // 如果失败
    return -1; // 表示失败

return 0; // 成功执行
}


int init_mysql() { // 初始化连接
// init the database connection
g_conn = mysql_init(NULL);

/* connect the database */
if(!mysql_real_connect(g_conn, g_host_name, g_user_name, g_password, g_db_name, g_db_port, NULL, 0)) // 如果失败
    return -1;

// 是否连接已经可用
if (executesql("set names utf8")) // 如果失败
    return -1;

return 0; // 返回成功
}

int main(int argc, char **argv) {
cJSON *pJson,*pSub,*dJson;
int iCount=0;
char noanswer_dest[64];
/*
memset(g_host_name, 0, sizeof(g_host_name));
memset(g_user_name, 0, sizeof(g_user_name));
memset(g_password, 0, sizeof(g_password));
memset(g_db_name, 0, sizeof(g_db_name));

GetProfileString(MYSQL_CONNECT_CONF, "general", "dbserverip", g_host_name);
GetProfileString(MYSQL_CONNECT_CONF, "general", "dbuser", g_user_name);
GetProfileString(MYSQL_CONNECT_CONF, "general", "dbpasswd", g_password);
GetProfileString(MYSQL_CONNECT_CONF, "general", "dbname", g_db_name);
*/
strcpy(g_host_name,getenv("MYSQL"));
strcpy(g_user_name,getenv("MYSQL_USER"));
strcpy(g_password,getenv("MYSQL_PASSWORD"));
strcpy(g_db_name,getenv("MYSQL_DATABASE"));
if (init_mysql()){
    print_mysql_error(NULL);
    exit(1);
}

if (executesql(QUERY_PAGING_USER_SQL)){
    print_mysql_error(NULL);
    exit(1);
}

g_res = mysql_store_result(g_conn); // 从服务器传送结果集至本地，mysql_use_result直接使用服务器上的记录集
FILE *conf_user_queue_fp = fopen(EXTEN_USER_QUEUE_FILE, "w+");
FILE *conf_queues_fp = fopen(QUEUES_FILE, "w+");

if (conf_user_queue_fp == NULL){
    perror("Open paging conf file Error: ");
    exit(1);
}

    fprintf(conf_user_queue_fp, ";!\n\
;! Automatically generated configuration file\n\
;! Filename: extensions_users_queue_custom.conf (/etc/asterisk/extensions_users_queue_custom.conf)\n\
;! Generator: Generator Paging\n\
;! Creation Date: %s\n\
;!\n\n\
",\
mytime()\
);

if (conf_queues_fp == NULL){
    perror("Open paging conf file Error: ");
    exit(1);
}

    fprintf(conf_queues_fp, ";!\n\
;! Automatically generated configuration file\n\
;! Filename: queues.conf (/etc/asterisk/queues.conf)\n\
;! Generator: Generator Paging\n\
;! Creation Date: %s\n\
;!\n\n\
\n\
[general]\n\
persistentmembers = yes\n\
\n\
",\
mytime()\
);

while ((g_row=mysql_fetch_row(g_res))){ // 打印结果集
    if (g_row[0] == NULL || g_row[1] == NULL || g_row[2] == NULL){
        printf("some feild is empty!\n");
        continue;
    }
    if(g_row[1] != NULL){
        pJson = cJSON_Parse(g_row[1]);
        iCount = cJSON_GetArraySize(pJson);
        if(iCount > 0){
            int q = 100000 + atoi(g_row[0]);
            fprintf(conf_user_queue_fp, "[manager-queue-%d]\n", q);
            fprintf(conf_queues_fp, "\
[Q%d]\n\
setinterfacevar = yes\n\
setqueueentryvar = yes\n\
strategy = %s\n\
timeout = 30\n\
wrapuptime = 0\n\
autofill = yes\n\
autopause = no\n\
ringinuse = no\n\
maxlen = 8\n\
context = queue-custom\n\
joinempty = no\n\
leavewhenempty = paused,unavailable,invalid,unknown\n\
periodic-announce-frequency = 0\n\
reportholdtime = no\n\
announce-frequency = 0\n\
announce-holdtime = no\n\
announce-position = no\n\
queue-youarenext =\n\
queue-callswaiting =\n\
queue-holdtime =\n\
queue-minutes =\n\
queue-thankyou =\n\
musicclass = queuemusic\n\
\n",q,g_row[2]);
            dJson = cJSON_Parse(g_row[4]);
            memset(noanswer_dest,0,sizeof(noanswer_dest));
            if(dJson)
            {
                if(strcmp(cJSON_GetObjectItem(dJson, "type")->valuestring, "hangup") == 0){
                    strcpy(noanswer_dest,"Goto(hangup,s,1)");
                }
                else if(strcmp(cJSON_GetObjectItem(dJson, "type")->valuestring, "extension") == 0){
                    sprintf(noanswer_dest,"Goto(default,%s,1)",cJSON_GetObjectItem(dJson, "exten")->valuestring);
                }
                else if(strcmp(cJSON_GetObjectItem(dJson, "type")->valuestring, "user") == 0){
                    int id = 100000 + cJSON_GetObjectItem(dJson, "id")->valueint;
                    sprintf(noanswer_dest,"Goto(manager-queue-%d,s,1)",id);
                }
                else if(strcmp(cJSON_GetObjectItem(dJson, "type")->valuestring, "outcall") == 0){
                    sprintf(noanswer_dest,"Goto(CallingRule_OutCall,%s,1)",cJSON_GetObjectItem(dJson, "exten")->valuestring);
                }
            }
            else
            {
                strcpy(noanswer_dest,"Goto(hangup,s,1)");
            }
            printf("parse success\n");
            if(iCount > 0)
            {
                if(strcmp(g_row[2],"ringall") == 0){
                    for(int i = 0;i < iCount;i++){
                        pSub = cJSON_GetArrayItem(pJson,i);
                        if(pSub != NULL){
                            fprintf(conf_queues_fp, "member => SIP/%s\n",pSub->valuestring);
                            fprintf(conf_user_queue_fp, "exten => %s,1,Macro(queue,Q%d,${EXTEN},%s)\n", pSub->valuestring,q,g_row[3]);
                            fprintf(conf_user_queue_fp, "same => n,%s\n", noanswer_dest);
                        }
                    }
                    fprintf(conf_user_queue_fp, "exten => s,1,Macro(queue,Q%d,%s,%s)\n", q, pSub->valuestring,g_row[3]);
                    fprintf(conf_user_queue_fp, "same => n,%s\n", noanswer_dest);
                }else{
                    for(int i = 0;i < iCount;i++){
                        pSub = cJSON_GetArrayItem(pJson,i);
                        if(pSub != NULL){
                            fprintf(conf_queues_fp, "member => SIP/%s,%d\n",pSub->valuestring,i);
                            fprintf(conf_user_queue_fp, "exten => %s,1,Macro(queue,Q%d,${EXTEN},%s)\n", pSub->valuestring,q,g_row[3]);
                            fprintf(conf_user_queue_fp, "same => n,%s\n", noanswer_dest);
                        }
                    }
                    fprintf(conf_user_queue_fp, "exten => s,1,Macro(queue,Q%d,%s,%s)\n", q, pSub->valuestring,g_row[3]);
                    fprintf(conf_user_queue_fp, "same => n,%s\n", noanswer_dest);
                }
            }
        }
    }
}
fclose(conf_queues_fp);
fclose(conf_user_queue_fp);
mysql_free_result(g_res); // 释放结果集
mysql_free_result(d_res);
mysql_close(g_conn); // 关闭链接
cJSON_Delete(dJson);
cJSON_Delete(pJson);
}